@@ -24,11 +24,7 @@ import json
 import textwrap
 import collections
 from dendropy.dataio import ioservice
-
-try:
-    from StringIO import StringIO # Python 2 legacy support: StringIO in this module is the one needed (not io)
-except ImportError:
-    from io import StringIO # Python 3
+from dendropy.utility.textprocessing import StringIO
 
 ############################################################################
 ## Local Module Methods
@@ -98,17 +94,17 @@ class NexmlWriter(ioservice.DataWriter):
         -----------------
 
         markup_as_sequences : boolean
-            If `True`, then character data will be marked up as sequences
-            instead of individual cells. Defaults to `False`.
-        suppress_unreferenced_taxon_namespaces: boolean, default: `False`
-            If `True`, then when writing |DataSet| objects, any
+            If |True|, then character data will be marked up as sequences
+            instead of individual cells. Defaults to |False|.
+        suppress_unreferenced_taxon_namespaces: boolean, default: |False|
+            If |True|, then when writing |DataSet| objects, any
             |TaxonNamespace| object in the DataSet's ``taxon_namespaces``
             collection will *not* be written as a "TAXA" block if it is not
             referenced by any character matrix (``char_matrices``) or tree list
             (``tree_lists``).
-        ignore_unrecognized_keyword_arguments : boolean, default: `False`
-            If `True`, then unsupported or unrecognized keyword arguments will
-            not result in an error. Default is `False`: unsupported keyword
+        ignore_unrecognized_keyword_arguments : boolean, default: |False|
+            If |True|, then unsupported or unrecognized keyword arguments will
+            not result in an error. Default is |False|: unsupported keyword
             arguments will result in an error.
 
         """
@@ -337,7 +333,7 @@ class NexmlWriter(ioservice.DataWriter):
                         print_count += 1
                 dest.write("\n{}</seq>\n".format(self.indent * (indent_level+3)))
             else:
-                for col_idx, (char_value, cell_char_type, cell_annotations) in enumerate(char_vector.iter_cells()):
+                for col_idx, (char_value, cell_char_type, cell_annotations) in enumerate(char_vector.cell_iter()):
                     parts = []
                     parts.append('%s<cell' % (self.indent*(indent_level+3)))
                     parts.append('char="%s"' % cell_char_type_id_map[ (taxon, col_idx) ])
@@ -592,7 +588,7 @@ class NexmlWriter(ioservice.DataWriter):
         char_type_ids_written = set()
         for taxon in char_matrix:
             char_vector = char_matrix[taxon]
-            for col_idx, (char_value, cell_char_type, cell_annotations) in enumerate(char_vector.iter_cells()):
+            for col_idx, (char_value, cell_char_type, cell_annotations) in enumerate(char_vector.cell_iter()):
                 if cell_char_type is None:
                     if char_matrix.data_type == "continuous":
                         char_type_id, char_type_xml = self._compose_char_type_xml_for_continuous_type(indent_level=indent_level+1)
