@@ -23,10 +23,8 @@ Infrastructure for phylogenetic data objects.
 import os
 import copy
 import sys
-try:
-    from StringIO import StringIO # Python 2 legacy support: StringIO in this module is the one needed (not io)
-except ImportError:
-    from io import StringIO # Python 3
+import collections
+from dendropy.utility.textprocessing import StringIO
 if not (sys.version_info.major >= 3 and sys.version_info.minor >= 4):
     from dendropy.utility.filesys import pre_py34_open as open
 from dendropy.utility import container
@@ -696,7 +694,7 @@ class Annotable(object):
             ``other``) and ``id(x)`` is found in ``attribute_object_mapper``,
             then in the copy the owner of the attribute is changed to
             ``attribute_object_mapper[id(x)]``.
-            If ``attribute_object_mapper`` is `None` (default), then the
+            If ``attribute_object_mapper`` is |None| (default), then the
             following mapping is automatically inserted: ``id(other): self``.
             That is, any references to ``other`` in any |Annotation|
             object will be remapped to ``self``.  If really no reattribution
@@ -733,7 +731,8 @@ class Annotable(object):
                 if a2.is_attribute and a1._value[0] is other:
                     a2._value = (self, a1._value[1])
                 self.annotations.add(a2)
-            memo[id(other._annotations)] = self._annotations
+            if hasattr(self, "_annotations"):
+                memo[id(other._annotations)] = self._annotations
 
     # def __copy__(self):
     #     o = self.__class__.__new__(self.__class__)
@@ -977,7 +976,7 @@ class AnnotationSet(container.OrderedSet):
         try:
             o = self.__class__(target=memo[id(self.target)])
         except KeyError:
-            raise KeyError("deepcopy error: object id {} not found: {}".format((id(self.target), repr(self.target))))
+            raise KeyError("deepcopy error: object id {} not found: {}".format(id(self.target), repr(self.target)))
         memo[id(self)] = o
         for a in self:
             x = copy.deepcopy(a, memo)
@@ -993,8 +992,9 @@ class AnnotationSet(container.OrderedSet):
             return container.OrderedSet.__getitem__(self, name)
         for a in self:
             if a.name == name:
-                return a.value
-        return self.add_new(name, "")
+                return a
+        a = self.add_new(name, "")
+        return a
 
     def __setitem__(self, name, value):
         """
@@ -1368,9 +1368,9 @@ class AnnotationSet(container.OrderedSet):
 
         Returns
         -------
-        results : |AnnotationSet| or `None`
+        results : |AnnotationSet| or |None|
             |AnnotationSet| containing |Annotation| objects that
-            match criteria, or `None` if no matching annotations found.
+            match criteria, or |None| if no matching annotations found.
         """
         results = []
         for a in self:
@@ -1394,9 +1394,9 @@ class AnnotationSet(container.OrderedSet):
 
         Returns
         -------
-        results : |Annotation| or `None`
+        results : |Annotation| or |None|
             First |Annotation| object found that matches criteria, or
-            `None` if no matching annotations found.
+            |None| if no matching annotations found.
         """
         if "default" in kwargs:
             default = kwargs["default"]
@@ -1427,9 +1427,9 @@ class AnnotationSet(container.OrderedSet):
 
         Returns
         -------
-        results : |Annotation| or `None`
+        results : |Annotation| or |None|
             ``value`` of first |Annotation| object found that matches
-            criteria, or `None` if no matching annotations found.
+            criteria, or |None| if no matching annotations found.
         """
         for a in self:
             if a.is_match(name=name):
@@ -1450,7 +1450,7 @@ class AnnotationSet(container.OrderedSet):
 
         Returns
         -------
-        results : |Annotation| or `None`
+        results : |Annotation| or |None|
             ``value`` of first |Annotation| object found that matches
             criteria.
         """
