@@ -23,15 +23,12 @@ that manages collections of |TaxonNamespace|, |TreeList|, and
 """
 
 import warnings
-try:
-    from StringIO import StringIO # Python 2 legacy support: StringIO in this module is the one needed (not io)
-except ImportError:
-    from io import StringIO # Python 3
 import copy
 import sys
 from dendropy.utility import container
 from dendropy.utility import error
 from dendropy.utility import deprecate
+from dendropy.utility import textprocessing
 from dendropy.datamodel import basemodel
 from dendropy.datamodel import taxonmodel
 from dendropy.datamodel import treecollectionmodel
@@ -156,16 +153,16 @@ class DataSet(
 
         **Optional General Keyword Arguments:**
 
-            - **exclude_trees** (*bool*) -- If ``True``, then all tree data in the data
+            - **exclude_trees** (*bool*) -- If |True|, then all tree data in the data
               source will be skipped.
-            - **exclude_chars** (*bool*) -- If ``True``, then all character
+            - **exclude_chars** (*bool*) -- If |True|, then all character
               data in the data source will be skipped.
             - **taxon_namespace** (|TaxonNamespace|) -- The |TaxonNamespace|
               instance to use to :doc:`manage the taxon names </primer/taxa>`.
               If not specified, a new one will be created.
-            - **ignore_unrecognized_keyword_arguments** (*bool*) -- If `True`,
+            - **ignore_unrecognized_keyword_arguments** (*bool*) -- If |True|,
               then unsupported or unrecognized keyword arguments will not
-              result in an error. Default is `False`: unsupported keyword
+              result in an error. Default is |False|: unsupported keyword
               arguments will result in an error.
 
         **Optional Schema-Specific Keyword Arguments:**
@@ -305,19 +302,19 @@ class DataSet(
 
         **Optional General Keyword Arguments:**
 
-            - **exclude_trees** (*bool*) -- If ``True``, then all tree data in the data
+            - **exclude_trees** (*bool*) -- If |True|, then all tree data in the data
               source will be skipped.
-            - **exclude_chars** (*bool*) -- If ``True``, then all character
+            - **exclude_chars** (*bool*) -- If |True|, then all character
               data in the data source will be skipped.
             - **taxon_namespace** (|TaxonNamespace|) -- The |TaxonNamespace|
               instance to use to :doc:`manage the taxon names </primer/taxa>`.
               If not specified, a new one will be created unless the DataSet
               object is in attached taxon namespace mode
-              (``self.attached_taxon_namespace`` is not ``None`` but assigned
+              (``self.attached_taxon_namespace`` is not |None| but assigned
               to a specific |TaxonNamespace| instance).
-            - **ignore_unrecognized_keyword_arguments** (*bool*) -- If `True`,
+            - **ignore_unrecognized_keyword_arguments** (*bool*) -- If |True|,
               then unsupported or unrecognized keyword arguments will not
-              result in an error. Default is `False`: unsupported keyword
+              result in an error. Default is |False|: unsupported keyword
               arguments will result in an error.
 
         **Optional Schema-Specific Keyword Arguments:**
@@ -642,7 +639,7 @@ class DataSet(
                     repr(self.attached_taxon_namespace), repr(kwargs["taxon_namespace"])))
             else:
                 kwargs["taxon_namespace"] = self.attached_taxon_namespace
-        if isinstance(char_matrix_type, str):
+        if textprocessing.is_str_type(char_matrix_type):
             char_matrix = charmatrixmodel.new_char_matrix(
                     data_type=char_matrix_type,
                     *args,
