@@ -26,12 +26,9 @@ import subprocess
 import tempfile
 import re
 import csv
-try:
-    from StringIO import StringIO # Python 2 legacy support: StringIO in this module is the one needed (not io)
-except ImportError:
-    from io import StringIO # Python 3
 
 import dendropy
+from dendropy.utility import textprocessing
 from dendropy.utility import error
 from dendropy.utility import metavar
 from dendropy.utility import container
@@ -63,7 +60,8 @@ class PaupService(object):
             strip_extraneous_prompts_from_stderr=True,
             cwd=None,
             env=None,
-            paup_path=PAUP_PATH
+            paup_path=PAUP_PATH,
+            timeout=None,
             ):
         """
         Executes a sequence of commands in PAUP* and returns the results.
@@ -74,19 +72,19 @@ class PaupService(object):
             A list or some other iterable of strings representing PAUP
             commands.
         suppress_standard_preamble : bool
-            If `True`, then the command sequence will not be prefaced by the
+            If |True|, then the command sequence will not be prefaced by the
             standard preamble.
         ignore_error_returncode : bool
-            If `True`, then a non-0 return code from the PAUP process will not
+            If |True|, then a non-0 return code from the PAUP process will not
             result in an exception being raised.
         ignore_nonempty_stderr : bool
-            If `True`, then the PAUP process writing to standard error will not
+            If |True|, then the PAUP process writing to standard error will not
             result in an exception being raised.
         strip_extraneous_prompts_from_stdout : bool
-            If `True`, then all occurrences of 'paup>' will be removed from the
+            If |True|, then all occurrences of 'paup>' will be removed from the
             standard output contents.
         strip_extraneous_prompts_from_stderr : bool
-            If `True`, then all occurrences of 'paup>' will be removed from the
+            If |True|, then all occurrences of 'paup>' will be removed from the
             standard error contents.
         cwd : string
             Set the working directory of the PAUP* process to this directory.
@@ -103,7 +101,7 @@ class PaupService(object):
         stderr : string
             Contents of the PAUP process standard error.
         """
-        if isinstance(paup_commands, str):
+        if textprocessing.is_str_type(paup_commands):
             commands = [paup_commands]
         else:
             commands = list(paup_commands)
@@ -120,7 +118,15 @@ class PaupService(object):
                 cwd=cwd,
                 env=env,
                 )
-        raw_stdout, raw_stderr = processio.communicate(p, paup_block)
+        raw_stdout, raw_stderr = processio.communicate(p, paup_block, timeout=timeout)
+        # try:
+        #     raw_stdout, raw_stderr = processio.communicate(p, paup_block, timeout=timeout)
+        # except TypeError as e:
+        #     raise
+        #     if str(e) == "communicate() got an unexpected keyword argument 'timeout'":
+        #         raw_stdout, raw_stderr = processio.communicate(p, paup_block)
+        #     else:
+        #         raise
         stdout = raw_stdout
         stderr = raw_stderr
         if strip_extraneous_prompts_from_stdout:
@@ -197,11 +203,11 @@ class PaupService(object):
             A list or some other iterable of file paths containing trees in
             NEXUS format.
         is_rooted : bool
-            If `True` then trees will be treated as rooted. If `False`, then
+            If |True| then trees will be treated as rooted. If |False|, then
             rooting follows that specified in the tree statements, defaulting
             to unrooted if not specified.
         use_tree_weights : bool
-            If `False` then tree weighting statements are disregarded.
+            If |False| then tree weighting statements are disregarded.
             Otherwise, they will be regarded.
         burnin : integer
             Skip these many trees (from beginning of each source).
@@ -273,11 +279,11 @@ class PaupService(object):
             A list or some other iterable of file paths containing trees in
             NEXUS format.
         is_rooted : bool
-            If `True` then trees will be treated as rooted. If `False`, then
+            If |True| then trees will be treated as rooted. If |False|, then
             rooting follows that specified in the tree statements, defaulting
             to unrooted if not specified.
         use_tree_weights : bool
-            If `False` then tree weighting statements are disregarded.
+            If |False| then tree weighting statements are disregarded.
             Otherwise, they will be regarded.
         burnin : integer
             Skip these many trees (from beginning of each source).
@@ -334,7 +340,7 @@ class PaupService(object):
         Composes commands to load a set of trees into PAUP*, with the specified
         number of burnin dropped.
         """
-        if isinstance(tree_filepaths, str):
+        if textprocessing.is_str_type(tree_filepaths):
             raise Exception("expecting list of filepaths, not string")
         if is_rooted is None:
             rooting = ""
@@ -686,7 +692,10 @@ def estimate_tree(char_matrix,
                     prop_invar=True,
                     extra_pre_est_commands=None,
                     extra_post_est_commands=None,
-                    paup_path='paup'):
+                    paup_path='paup',
+                    char_matrix_writing_kwargs=None,
+                    timeout=None,
+                    ):
     """
     Given a dataset, ``char_matrix``, estimates a tree using the given criterion.
     """
@@ -697,7 +706,9 @@ def estimate_tree(char_matrix,
         'pinvar' : prop_invar and 'estimate' or '0',
     }
     cf = tempfile.NamedTemporaryFile("w", delete=True)
-    char_matrix.write_to_stream(cf, schema='nexus')
+    if not char_matrix_writing_kwargs:
+        char_matrix_writing_kwargs = {}
+    char_matrix.write_to_stream(cf, schema='nexus', **char_matrix_writing_kwargs)
     cf.flush()
     paup_args['datafile'] = cf.name
     # output_tree_file_handle, output_tree_filepath = tempfile.mkstemp(text=True)
@@ -705,13 +716,13 @@ def estimate_tree(char_matrix,
     output_tree_filepath = output_tree_file_handle.name
     paup_args['est_tree_file'] = output_tree_filepath
     if extra_pre_est_commands:
-        if isinstance(extra_pre_est_commands, str):
+        if textprocessing.is_str_type(extra_pre_est_commands):
             extra_pre_est_commands = [extra_pre_est_commands]
         paup_args["pre_est_commands"] = ";\n".join(extra_pre_est_commands)
     else:
         paup_args["pre_est_commands"] = ""
     if extra_post_est_commands:
-        if isinstance(extra_post_est_commands, str):
+        if textprocessing.is_str_type(extra_post_est_commands):
             extra_post_est_commands = [extra_post_est_commands]
         paup_args["post_est_commands"] = ";\n".join(extra_post_est_commands)
     else:
@@ -741,11 +752,16 @@ def estimate_tree(char_matrix,
     %(post_est_commands)s;
     savetrees file=%(est_tree_file)s format=nexus root=yes brlens=yes taxablk=yes maxdecimals=20;
     """
-    paup_run = subprocess.Popen(['%s -n' % paup_path],
-                                shell=True,
-                                stdin=subprocess.PIPE,
-                                stdout=subprocess.PIPE)
-    stdout, stderr = processio.communicate(paup_run, paup_template % paup_args)
+    # paup_run = subprocess.Popen(['%s -n' % paup_path],
+    #                             shell=True,
+    #                             stdin=subprocess.PIPE,
+    #                             stdout=subprocess.PIPE)
+    # stdout, stderr = processio.communicate(paup_run, paup_template % paup_args)
+    returncode, stdout, stderr = PaupService.call(
+            paup_commands=paup_template % paup_args,
+            paup_path=paup_path,
+            timeout=timeout,
+            )
     t = dendropy.Tree.get_from_path(output_tree_filepath, "nexus", taxon_namespace=char_matrix.taxon_namespace)
     cf.close()
     output_tree_file_handle.close()
