@@ -26,6 +26,7 @@ import dendropy
 from dendropy.utility import GLOBAL_RNG
 from dendropy.utility import constants
 from dendropy.calculate import probability
+from dendropy.calculate import combinatorics
 
 ###############################################################################
 ## Calculations and statistics
@@ -52,7 +53,7 @@ def discrete_time_to_coalescence(n_genes,
     n_to_coalesce : integer
         The waiting time that will be returned will be the waiting time for
         this number of genes in the sample to coalesce.
-    rng : `Random`
+    rng : ``Random`` object
         The random number generator instance.
 
     Returns
@@ -69,7 +70,7 @@ def discrete_time_to_coalescence(n_genes,
         time_units = pop_size
     if rng is None:
         rng = GLOBAL_RNG
-    p = pop_size / probability.binomial_coefficient(n_genes, n_to_coalesce)
+    p = pop_size / combinatorics.choose(n_genes, n_to_coalesce)
     tmrca = probability.geometric_rv(p)
     return tmrca * time_units
 
@@ -85,7 +86,7 @@ def time_to_coalescence(n_genes,
 
     Given the number of gene lineages in a sample, ``n_genes``, and a
     population size, ``pop_size``, this function returns a random number from
-    an exponential distribution with rate $\choose(``pop_size``, 2)$.
+    an exponential distribution with rate $\\choose(``pop_size``, 2)$.
     ``pop_size`` is the effective *haploid* population size; i.e., number of gene
     in the population: 2 * N in a diploid population of N individuals,
     or N in a haploid population of N individuals. If ``pop_size`` is 1 or 0 or
@@ -96,11 +97,11 @@ def time_to_coalescence(n_genes,
     The coalescence time, or the waiting time for the coalescence, of two
     gene lineages evolving in a population with haploid size $N$ is an
     exponentially-distributed random variable with rate of $N$ an
-    expectation of $\frac{1}{N}$).
+    expectation of $\\frac{1}{N}$).
     The waiting time for coalescence of *any* two gene lineages in a sample of
     $n$ gene lineages evolving in a population with haploid size $N$ is an
-    exponentially-distributed random variable with rate of $\choose{N, 2}$ and
-    an expectation of $\frac{1}{\choose{N, 2}}$.
+    exponentially-distributed random variable with rate of $\\choose{N, 2}$ and
+    an expectation of $\\frac{1}{\choose{N, 2}}$.
 
     Parameters
     ----------
@@ -113,7 +114,7 @@ def time_to_coalescence(n_genes,
     n_to_coalesce : integer
         The waiting time that will be returned will be the waiting time for
         this number of genes in the sample to coalesce.
-    rng : `Random`
+    rng : ``Random`` object
         The random number generator instance to use.
 
     Returns
@@ -129,7 +130,7 @@ def time_to_coalescence(n_genes,
         time_units = 1.0
     else:
         time_units = pop_size
-    rate = probability.binomial_coefficient(n_genes, n_to_coalesce)
+    rate = combinatorics.choose(n_genes, n_to_coalesce)
     tmrca = rng.expovariate(rate)
     return tmrca * time_units
 
@@ -150,7 +151,7 @@ def expected_tmrca(n_genes, pop_size=None, n_to_coalesce=2):
     n_to_coalesce : integer
         The waiting time that will be returned will be the waiting time for
         this number of genes in the sample to coalesce.
-    rng : `Random`
+    rng : ``Random`` object
         The random number generator instance.
 
     Returns
@@ -161,7 +162,7 @@ def expected_tmrca(n_genes, pop_size=None, n_to_coalesce=2):
         ``pop_size`` genes.
 
     """
-    nc2 = probability.binomial_coefficient(n_genes, n_to_coalesce)
+    nc2 = combinatorics.choose(n_genes, n_to_coalesce)
     tmrca = (float(1)/nc2)
     if pop_size is not None:
         return tmrca * pop_size
@@ -216,11 +217,11 @@ def coalesce_nodes(nodes,
         time equals 2N generations for a diploid population of size N, or N
         generations for a haploid population of size N. Otherwise time is in
         generations.
-    rng : `Random`
+    rng : ``Random`` object
         The random number generator instance to use. If not specified, the
         default RNG will be used.
     use_expected_tmrca : bool
-        If `True`, then instead of random times, the *expected* times will be
+        If |True|, then instead of random times, the *expected* times will be
         used.
 
     Returns
@@ -376,7 +377,7 @@ def extract_coalescent_frames(tree, ultrametricity_precision=constants.DEFAULT_U
 
     Returns
     -------
-    x : list of tuples (node, coalescent interval)
+    x : dict
         Returns dictionary, with key = number of alleles, and values = waiting
         time for coalescent for the given tree
     """
@@ -387,9 +388,7 @@ def extract_coalescent_frames(tree, ultrametricity_precision=constants.DEFAULT_U
     for n in nwti:
         num_genes_wt[num_genes] = n[1]
         num_genes = num_genes - len(n[0].child_nodes()) + 1
-
-    import sys
-    num_alleles_list = sorted(num_genes_wt.keys(), reverse=True)
+    # num_alleles_list = sorted(num_genes_wt.keys(), reverse=True)
     return num_genes_wt
 
 def log_probability_of_coalescent_frames(coalescent_frames, haploid_pop_size):
@@ -398,9 +397,11 @@ def log_probability_of_coalescent_frames(coalescent_frames, haploid_pop_size):
     Kingman1982b}, the waiting times between coalescent events in a
     sample of $k$ alleles segregating in a  population of (haploid) size
     $N_e$ is distributed exponentially with a rate parameter of
-    $\frac{{k \choose 2}}{N_e}$::
+    :math`\\frac{{k \choose 2}}{N_e}`::
 
-         \Pr(T) =  \frac{{k \choose 2}}{N_e} \e{-  \frac{{k \choose 2}}{N_e} T},
+        .. math::
+
+            \\Pr(T) =  \\frac{{k \\choose 2}}{N_e} \\e{-  \\frac{{k \\choose 2}}{N_e} T},
 
     where $T$ is the length of  (chronological) time in which there are
     $k$ alleles in the sample (i.e., for $k$ alleles to coalesce into
@@ -409,7 +410,7 @@ def log_probability_of_coalescent_frames(coalescent_frames, haploid_pop_size):
     lp = 0.0
     for k, t in coalescent_frames.items():
         k2N = (float(k * (k-1)) / 2) / haploid_pop_size
-#         k2N = float(probability.binomial_coefficient(k, 2)) / haploid_pop_size
+#         k2N = float(combinatorics.choose(k, 2)) / haploid_pop_size
         lp =  lp + math.log(k2N) - (k2N * t)
     return lp
 
