@@ -23,6 +23,7 @@ Various data structures.
 import collections
 import copy
 import sys
+import csv
 
 ###############################################################################
 ## OrderedSet
@@ -145,7 +146,7 @@ class OrderedSet(object):
 
     def __contains__(self, value):
         """
-        Returns `True` if ``value`` is in ``self`` or `False` otherwise.
+        Returns |True| if ``value`` is in ``self`` or |False| otherwise.
         """
         return value in self._item_set
 
@@ -676,6 +677,272 @@ class FrozenOrderedDict(collections.OrderedDict):
         temp._is_frozen = True
         return temp
 
+##############################################################################
+## DataTable
+
+class DataTable(object):
+
+    @classmethod
+    def from_csv(cls,
+            src,
+            is_first_row_column_names=True,
+            is_first_column_row_names=True,
+            is_trim_space=True,
+            default_data_type=None,
+            column_data_types=None,
+            label_transform_fn=None,
+            **csv_reader_kwargs
+            ):
+        """
+        Returns table from a full-configured csv.Reader instance.
+
+        Parameters
+        -----------
+        csv_reader : a csv.Reader instance
+            The source of the data.
+        is_first_row_column_names : bool
+            If True, then the first row is interpreted as column names;
+            otherwise, treated as data row.
+        is_first_column_row_names : bool
+            If True, then the first column is interpreted as row names;
+            otherwise, treated as data column.
+        is_trim_space : bool
+            If True, then will strip space from both sides of all tokens before
+            processing. Note that only spaces will be trimmed, not tabs.
+        default_data_type : type or function object
+            Any callable that, when passed a value, returns the coerced-to-type
+            equivalent for the data.
+        column_data_types : dict
+            A dictionary where the key are the column names and the
+            corresponding value the type (see ``default_data_type`` for
+            description).
+
+        Returns
+        -------
+        t: a DataTable instance
+            Returns table from a full-configured csv.Reader instance.
+
+        """
+        if isinstance(src, str):
+            with open(src, "r") as fsrc:
+                return cls._from_csv_file(
+                    src=fsrc,
+                    is_first_row_column_names=is_first_row_column_names,
+                    is_first_column_row_names=is_first_column_row_names,
+                    is_trim_space=is_trim_space,
+                    default_data_type=default_data_type,
+                    column_data_types=column_data_types,
+                    label_transform_fn=label_transform_fn,
+                    **csv_reader_kwargs)
+        else:
+            return cls._from_csv_file(
+                src=src,
+                is_first_row_column_names=is_first_row_column_names,
+                is_first_column_row_names=is_first_column_row_names,
+                is_trim_space=is_trim_space,
+                default_data_type=default_data_type,
+                column_data_types=column_data_types,
+                label_transform_fn=label_transform_fn,
+                **csv_reader_kwargs)
+
+    @classmethod
+    def _from_csv_file(cls,
+            src,
+            is_first_row_column_names=True,
+            is_first_column_row_names=True,
+            is_trim_space=True,
+            default_data_type=None,
+            column_data_types=None,
+            label_transform_fn=None,
+            **csv_reader_kwargs
+            ):
+        ncols = None
+        data_table = cls()
+        if is_first_row_column_names:
+            first_data_row_offset = 1
+        else:
+            first_data_row_offset = 0
+        if is_first_column_row_names:
+            first_data_column_offset = 1
+        else:
+            first_data_column_offset = 0
+        if column_data_types is None:
+            column_data_types = {}
+        if label_transform_fn is None:
+            label_transform_fn = lambda x: x
+        csv_reader = csv.reader(src, **csv_reader_kwargs)
+        for row_idx, row in enumerate(csv_reader):
+            if ncols is None:
+                ncols = len(row)
+            else:
+                if len(row) == 1 and row[0].strip() == "" and is_trim_space: # blank row
+                    continue
+                assert ncols == len(row), "{} != {}".format(ncols, len(row))
+            for cell_idx, cell in enumerate(row):
+                if is_trim_space:
+                    cell = cell.strip(" ")
+                if row_idx == 0 and is_first_row_column_names:
+                    if cell_idx == 0 and is_first_column_row_names:
+                        continue
+                    data_table.add_column(
+                            column_name=label_transform_fn(cell),
+                            data_type=column_data_types.get(cell, default_data_type))
+                elif cell_idx == 0 and is_first_column_row_names:
+                    data_table.add_row(label_transform_fn(cell))
+                else:
+                    if row_idx == 0 and not is_first_row_column_names:
+                        data_table.add_column(data_type=column_data_types.get(cell, default_data_type))
+                    if cell_idx == 0 and not is_first_column_row_names:
+                        data_table.add_row()
+                    effective_row_idx = row_idx - first_data_row_offset
+                    assert effective_row_idx < len(data_table._row_names), "{}: {}".format(effective_row_idx, data_table._row_names)
+                    effective_column_idx = cell_idx - first_data_column_offset
+                    assert effective_column_idx < len(data_table._column_names), "{}: {}".format(effective_column_idx, data_table._column_names)
+                    data_table[effective_row_idx, effective_column_idx] = cell
+        return data_table
+
+    def __init__(self):
+        self._row_names = []
+        self._row_name_set = set()
+        self._column_names = []
+        self._column_data_types = {}
+        self._column_name_set = set()
+        self._data = {}
+
+    def add_column(self, column_name=None, pos=None, data_type=None):
+        column_name = self._validate_new_column_name(column_name)
+        assert column_name not in self._column_name_set
+        if pos is None:
+            pos = len(self._column_names)
+        self._column_names.insert(pos, column_name)
+        self._column_name_set.add(column_name)
+        self._column_data_types[column_name] = data_type
+
+    def add_row(self, row_name=None, pos=None):
+        row_name = self._validate_new_row_name(row_name)
+        assert row_name not in self._row_name_set
+        if pos is None:
+            pos = len(self._row_names)
+        self._row_names.insert(pos, row_name)
+        self._row_name_set.add(row_name)
+
+    def __getitem__(self, key):
+        row_name = self._dereference_key(
+                key=key[0],
+                name_list=self._row_names,
+                name_set=self._row_name_set)
+        column_name = self._dereference_key(
+                key=key[1],
+                name_list=self._column_names,
+                name_set=self._column_name_set)
+        if row_name not in self._data:
+            return None
+        if column_name not in self._data[row_name]:
+            return None
+        return self._data[row_name][column_name]
+
+    def __setitem__(self, key, value):
+        row_name = self._dereference_key(
+                key=key[0],
+                name_list=self._row_names,
+                name_set=self._row_name_set)
+        column_name = self._dereference_key(
+                key=key[1],
+                name_list=self._column_names,
+                name_set=self._column_name_set)
+        if row_name not in self._data:
+            self._data[row_name] = {}
+        if self._column_data_types[column_name] is not None:
+            value = self._column_data_types[column_name](value)
+        self._data[row_name][column_name] = value
+
+    def row_name_iter(self):
+        for row_name in self._row_names:
+            yield row_name
+
+    def column_name_iter(self):
+        for column_name in self._column_names:
+            yield column_name
+
+    def row_value_iter(self, column_name):
+        column_name = self._dereference_key(
+                key=column_name,
+                name_list=self._column_names,
+                name_set=self._column_name_set)
+        for row_name in self._row_names:
+            yield self[row_name, column_name]
+
+    def column_value_iter(self, row_name):
+        row_name = self._dereference_key(
+                key=row_name,
+                name_list=self._row_names,
+                name_set=self._row_name_set)
+        for column_name in self._column_names:
+            yield self[row_name, column_name]
+
+    def write_csv(self,
+            out,
+            is_first_row_column_names=True,
+            is_first_column_row_names=True,
+            missing_data_value="NA",
+            **csv_writer_kwargs
+            ):
+        if isinstance(out, str):
+            dest = open(out, "w")
+        else:
+            dest = out
+        if "delimiter" not in csv_writer_kwargs:
+            csv_writer_kwargs["delimiter"] = ","
+        csv_writer = csv.writer(dest, csv_writer_kwargs)
+        if is_first_row_column_names:
+            header = []
+            if is_first_column_row_names:
+                header.append("")
+            header.extend(self._column_names)
+            csv_writer.writerow(header)
+        for row_name in self._row_names:
+            row = []
+            if is_first_column_row_names:
+                row.append(row_name)
+            for column_name in self._column_names:
+                v = self[row_name, column_name]
+                if v is None:
+                    v = missing_data_value
+                row.append(v)
+            csv_writer.writerow(row)
+
+    def _validate_new_column_name(self, column_name=None):
+        if column_name is None:
+            column_name = "V{}".format(len(self._column_names))
+        else:
+            column_name = str(column_name)
+        return column_name
+
+    def _validate_new_row_name(self, row_name=None):
+        if row_name is None:
+            row_name = "V{}".format(len(self._row_names))
+        else:
+            row_name = str(row_name)
+        return row_name
+
+    def _dereference_key(self, key, name_list, name_set):
+        if isinstance(key, int):
+            return name_list[key]
+        else:
+            if key not in name_set:
+                raise KeyError(key)
+            else:
+                return key
+
+    def num_rows(self):
+        return len(self._row_names)
+
+    def num_columns(self):
+        return len(self._column_names)
+
+    def __len__(self):
+        return self.num_rows()
+
 ###############################################################################
 ## Generic Container Interace (for reference)
 
